@@ -15,13 +15,14 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: EphyPromptService.cpp,v 1.2 2006/01/23 19:34:28 chpe Exp $
+ *  $Id: EphyPromptService.cpp,v 1.8 2006/02/13 23:07:52 chpe Exp $
  */
 
 #include "mozilla-config.h"
 #include "config.h"
 
 #include "EphyPromptService.h"
+#include "AutoJSContextStack.h"
 
 #include <nsCOMPtr.h>
 #include <nsIDOMWindow.h>
@@ -86,6 +87,7 @@ public:
 	void GetSelected (PRInt32*);
 
 	PRInt32 Run (PRBool * = nsnull);
+	void Show ();
 
 	PRBool IsCalledFromScript ();
 	void PerformScriptAbortion ();
@@ -482,6 +484,11 @@ Prompter::Run (PRBool *aSuccess)
 	}
 #endif
 
+	nsresult rv;
+	AutoJSContextStack stack;
+	rv = stack.Init ();
+	if (NS_FAILED (rv)) return rv;
+
 	if (mDelay)
 	{
 		guint timeout = g_timeout_add (TIMEOUT,
@@ -515,6 +522,29 @@ Prompter::Run (PRBool *aSuccess)
 	return mResponse;
 }
 
+static void
+DeletePrompter (gpointer aPromptPtr,
+	        GObject *aZombie)
+{
+	Prompter *prompt = NS_STATIC_CAST (Prompter*, aPromptPtr);
+
+	delete prompt;
+}
+
+void
+Prompter::Show ()
+{
+	gtk_window_set_modal (GTK_WINDOW (mDialog), FALSE);
+
+	g_signal_connect (mDialog, "response",
+			  G_CALLBACK (gtk_widget_destroy), NULL);
+	g_object_weak_ref (G_OBJECT (mDialog),
+			   (GWeakNotify) DeletePrompter,
+			   NS_STATIC_CAST (gpointer, this));
+
+	gtk_widget_show (GTK_WIDGET (mDialog));
+}
+
 PRBool
 Prompter::IsCalledFromScript()
 {
@@ -604,7 +634,14 @@ Prompter::ConvertAndEscapeButtonText(const PRUnichar *aText,
 }
 
 /* FIXME: needs THREADSAFE? */
-NS_IMPL_ISUPPORTS1 (EphyPromptService, nsIPromptService)
+#if HAVE_NSINONBLOCKINGALERTSERVICE_H
+NS_IMPL_ISUPPORTS2 (EphyPromptService,
+		    nsIPromptService,
+		    nsINonBlockingAlertService)
+#else
+NS_IMPL_ISUPPORTS1 (EphyPromptService,
+		    nsIPromptService)
+#endif
 
 EphyPromptService::EphyPromptService()
 {
@@ -756,10 +793,10 @@ EphyPromptService::PromptUsernameAndPassword (nsIDOMWindow *aParent,
 	NS_ENSURE_ARG_POINTER (aUsername);
 	NS_ENSURE_ARG_POINTER (aPassword);
 
-	Prompter prompt (GTK_STOCK_DIALOG_QUESTION, aParent, aDialogTitle, aText);
+	Prompter prompt (GTK_STOCK_DIALOG_AUTHENTICATION, aParent, aDialogTitle, aText);
 	prompt.AddStockButton (GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
 	prompt.AddStockButton (GTK_STOCK_OK, GTK_RESPONSE_ACCEPT);
-	prompt.AddEntry (_("_User:"), *aUsername, PR_FALSE);
+	prompt.AddEntry (_("_Username:"), *aUsername, PR_FALSE);
 	prompt.AddEntry (_("_Password:"), *aPassword, PR_TRUE);
 	prompt.AddCheckbox (aCheckMsg, aCheckState);
 
@@ -784,7 +821,7 @@ EphyPromptService::PromptPassword (nsIDOMWindow *aParent,
 	NS_ENSURE_ARG_POINTER (_retval);
 	NS_ENSURE_ARG_POINTER (aPassword);
 
-	Prompter prompt (GTK_STOCK_DIALOG_QUESTION, aParent, aDialogTitle, aText);
+	Prompter prompt (GTK_STOCK_DIALOG_AUTHENTICATION, aParent, aDialogTitle, aText);
 	prompt.AddStockButton (GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
 	prompt.AddStockButton (GTK_STOCK_OK, GTK_RESPONSE_ACCEPT);
 	prompt.AddEntry (_("_Password:"), *aPassword, PR_TRUE);
@@ -822,3 +859,22 @@ EphyPromptService::Select (nsIDOMWindow *aParent,
 
 	return RETVAL(response);
 }
+
+#if HAVE_NSINONBLOCKINGALERTSERVICE_H
+
+/* showNonBlockingAlert (in nsIDOMWindow aParent, in wstring aDialogTitle, in wstring aText); */
+NS_IMETHODIMP
+EphyPromptService::ShowNonBlockingAlert (nsIDOMWindow *aParent,
+					 const PRUnichar *aDialogTitle,
+					 const PRUnichar *aText)
+{
+	Prompter *prompt = new Prompter (GTK_STOCK_DIALOG_INFO, aParent, aDialogTitle, aText);
+	if (!prompt) return NS_ERROR_OUT_OF_MEMORY;
+
+	prompt->AddStockButton (GTK_STOCK_OK, GTK_RESPONSE_ACCEPT);
+	prompt->Show ();
+
+	return NS_OK;
+}
+
+#endif /* HAVE_NSINONBLOCKINGALERTSERVICE_H */
