@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: EphySingle.h,v 1.2 2004/06/27 11:53:34 chpe Exp $
+ *  $Id: EphySingle.h,v 1.4 2005/10/12 19:45:06 chpe Exp $
  */
 
 #ifndef EPHY_SINGLE_H
@@ -33,8 +33,7 @@
 #include <nsICookie.h>
 #include <nsIPermission.h>
 
-class EphySingle : public nsIObserver,
-		   public nsSupportsWeakReference
+class EphySingle : public nsIObserver
 {
 public:
 	NS_DECL_ISUPPORTS
@@ -46,6 +45,8 @@ public:
 	nsresult Init (EphyEmbedSingle *aOwner);
 	nsresult Detach ();
 
+	PRBool IsOnline() { return mIsOnline; }
+
 protected:
 	nsresult EmitCookieNotification (const char *name, nsISupports *aSubject);
 	nsresult EmitPermissionNotification (const char *name, nsISupports *aSubject);
@@ -53,6 +54,7 @@ protected:
 private:
 	nsCOMPtr<nsIObserverService> mObserverService;
 	EphyEmbedSingle *mOwner;
+	PRBool mIsOnline;
 };
 
 EphyCookie	   *mozilla_cookie_to_ephy_cookie	  (nsICookie *cookie);
