@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: GtkNSSDialogs.cpp,v 1.39 2005/12/05 20:23:49 chpe Exp $
+ *  $Id: GtkNSSDialogs.cpp,v 1.40 2006/02/02 22:25:18 chpe Exp $
  */
 
 /*
@@ -31,6 +31,7 @@
 #include "config.h"
 
 #include "EphyUtils.h"
+#include "AutoJSContextStack.h"
 
 #include <nsCOMPtr.h>
 #include <nsMemory.h>
@@ -207,6 +208,11 @@ display_cert_warning_box (nsIInterfaceRequestor *ctx,
 	GtkWidget *dialog, *label, *checkbox, *vbox, *button;
 	int res;
 
+	nsresult rv;
+	AutoJSContextStack stack;
+	rv = stack.Init ();
+	if (NS_FAILED (rv)) return rv;
+
 	/* NOTE: Due to a mozilla bug [https://bugzilla.mozilla.org/show_bug.cgi?id=306288],
 	 * we will always end up without a parent!
 	 */
@@ -539,6 +545,11 @@ GtkNSSDialogs::ConfirmDownloadCACert(nsIInterfaceRequestor *ctx,
 	GtkWidget *dialog, *label;
 	char *msg, *primary;
 
+	nsresult rv;
+	AutoJSContextStack stack;
+	rv = stack.Init ();
+	if (NS_FAILED (rv)) return rv;
+
 	nsCOMPtr<nsIDOMWindow> parent = do_GetInterface (ctx);
 	GtkWindow *gparent = GTK_WINDOW (EphyUtils::FindGtkParent (parent));
 
@@ -751,6 +762,11 @@ GtkNSSDialogs::SetPKCS12FilePassword(nsIInterfaceRequestor *ctx,
 	GtkWidget *progress;
 	char *msg;
 
+	nsresult rv;
+	AutoJSContextStack stack;
+	rv = stack.Init ();
+	if (NS_FAILED (rv)) return rv;
+
 	nsCOMPtr<nsIDOMWindow> parent = do_GetInterface (ctx);
 	GtkWindow *gparent = GTK_WINDOW (EphyUtils::FindGtkParent (parent));
 
@@ -869,6 +885,11 @@ GtkNSSDialogs::GetPKCS12FilePassword(nsIInterfaceRequestor *ctx,
 	GtkWidget *dialog, *hbox, *label, *entry, *vbox;
 	char *msg;
 
+	nsresult rv;
+	AutoJSContextStack stack;
+	rv = stack.Init ();
+	if (NS_FAILED (rv)) return rv;
+
 	nsCOMPtr<nsIDOMWindow> parent = do_GetInterface (ctx);
 	GtkWindow *gparent = GTK_WINDOW (EphyUtils::FindGtkParent (parent));
 
@@ -1313,11 +1334,15 @@ GtkNSSDialogs::ViewCert(nsIInterfaceRequestor *ctx,
 	GtkWidget *dialog, *widget;
 	GladeXML *gxml;
 	nsEmbedString value;
-	nsresult rv;
 	PRUint32 verifystate, count;
 	PRUnichar ** usage;
 	GtkSizeGroup * sizegroup;
 
+	nsresult rv;
+	AutoJSContextStack stack;
+	rv = stack.Init ();
+	if (NS_FAILED (rv)) return rv;
+
 	gxml = glade_xml_new (ephy_file ("certificate-dialogs.glade"),
 			      "viewcert_dialog", NULL);
 	g_return_val_if_fail (gxml != NULL, NS_ERROR_FAILURE);
