@@ -21,7 +21,7 @@
 
    Authors: Ramiro Estrugo <ramiro@eazel.com>
 
-   $Id: eel-gconf-extensions.h,v 1.4 2004/03/17 09:02:35 marco Exp $
+   $Id: eel-gconf-extensions.h,v 1.5 2006/01/16 23:51:40 chpe Exp $
 */
 
 #ifndef EEL_GCONF_EXTENSIONS_H
@@ -76,6 +76,8 @@ void         eel_gconf_set_float             (const char             *key,
 gfloat       eel_gconf_get_float             (const char             *key);
 void         eel_gconf_set_path              (const char             *key,
 					      const char             *value);
+void	     eel_gconf_unset_key	     (const char             *key);
+void	     eel_gconf_notify		     (const char             *key);
 
 G_END_DECLS
 
