@@ -1,7 +1,7 @@
 /*
  *  Copyright (C) 2002 Jorn Baayen
  *  Copyright (C) 2003, 2004 Marco Pesenti Gritti
- *  Copyright (C) 2004 Christian Persch
+ *  Copyright (C) 2004, 2005, 2006 Christian Persch
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-file-helpers.c,v 1.40 2006/01/16 23:51:40 chpe Exp $
+ *  $Id: ephy-file-helpers.c,v 1.47 2006/02/13 23:01:21 chpe Exp $
  */
 
 #include "config.h"
@@ -28,16 +28,13 @@
 #include "eel-gconf-extensions.h"
 #include "ephy-debug.h"
 
-#include <string.h>
-#include <stdlib.h>
-#include <unistd.h>
-#include <sys/stat.h>
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <libgnome/gnome-init.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <libgnomevfs/gnome-vfs-file-info.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
+#include <libgnomevfs/gnome-vfs-directory.h>
 #include <libxml/xmlreader.h>
 
 /* bug http://bugzilla.gnome.org/show_bug.cgi?id=156687 */
@@ -49,17 +46,29 @@
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
 
+#include <string.h>
+#include <stdlib.h>
+#include <unistd.h>
+#include <sys/stat.h>
+
 #define EPHY_UUID		"0d82d98f-7079-401c-abff-203fcde1ece3"
 #define EPHY_UUID_ENVVAR	"EPHY_UNIQUE"
 #define EPHY_UUID_ENVSTRING	EPHY_UUID_ENVVAR "=" EPHY_UUID
 
+#define DELAY_MAX_TICKS	64
+#define INITIAL_TICKS	2
+
 static GHashTable *files = NULL;
 static GHashTable *mime_table = NULL;
 
+static gboolean have_private_profile = FALSE;
+static gboolean keep_temp_directory = FALSE; /* for debug purposes */
 static char *dot_dir = NULL;
 static char *tmp_dir = NULL;
 static GList *del_on_exit = NULL;
 
+GQuark ephy_file_helpers_error_quark;
+
 const char *
 ephy_file_tmp_dir (void)
 {
@@ -246,19 +255,14 @@ ephy_file (const char *filename)
 const char *
 ephy_dot_dir (void)
 {
-	if (dot_dir == NULL)
-	{
-		dot_dir = g_build_filename (g_get_home_dir (),
-					    GNOME_DOT_GNOME,
-					    "epiphany",
-					    NULL);
-	}
-
 	return dot_dir;
 }
 
-void
-ephy_file_helpers_init (void)
+gboolean
+ephy_file_helpers_init (const char *profile_dir,
+			gboolean private_profile,
+			gboolean keep_temp_dir,
+			GError **error)
 {
 	const char *uuid;
 
@@ -273,10 +277,45 @@ ephy_file_helpers_init (void)
 	/* Put marker in env */
 	g_setenv (EPHY_UUID_ENVVAR, EPHY_UUID, TRUE);
 
+	ephy_file_helpers_error_quark = g_quark_from_static_string ("ephy-file-helpers-error");
+
 	files = g_hash_table_new_full (g_str_hash,
 				       g_str_equal,
 				       (GDestroyNotify) g_free,
 				       (GDestroyNotify) g_free);
+
+	have_private_profile = private_profile;
+	keep_temp_directory = keep_temp_dir;
+
+	if (private_profile && profile_dir != NULL)
+	{
+		dot_dir = g_strdup (profile_dir);
+	}
+	else if (private_profile)
+	{
+		if (ephy_file_tmp_dir () == NULL)
+		{
+			g_set_error (error,
+				     EPHY_FILE_HELPERS_ERROR_QUARK,
+				     0,
+				     _("Could not create a temporary directory in “%s”."),
+				     g_get_tmp_dir ());
+			return FALSE;
+		}
+
+		dot_dir = g_build_filename (ephy_file_tmp_dir (),
+					    "epiphany",
+					    NULL);
+	}
+	else
+	{
+		dot_dir = g_build_filename (g_get_home_dir (),
+					    GNOME_DOT_GNOME,
+					    "epiphany",
+					    NULL);
+	}
+	
+	return ephy_ensure_dir_exists (ephy_dot_dir (), error);
 }
 
 static void
@@ -306,33 +345,49 @@ ephy_file_helpers_shutdown (void)
 		mime_table = NULL;
 	}
 
+	g_free (dot_dir);
+	dot_dir = NULL;
+
 	if (tmp_dir != NULL)
 	{
-		rmdir (tmp_dir);
+		if (!keep_temp_directory)
+		{
+			/* recursively delete the contents */
+			ephy_file_delete_directory (tmp_dir);
+
+			/* delete the directory itself too */
+			rmdir (tmp_dir);
+		}
+
 		g_free (tmp_dir);
 		tmp_dir = NULL;
 	}
-
-	g_free (dot_dir);
-	dot_dir = NULL;
 }
 
 gboolean
-ephy_ensure_dir_exists (const char *dir)
+ephy_ensure_dir_exists (const char *dir,
+		        GError **error)
 {
-	if (g_file_test (dir, G_FILE_TEST_IS_DIR) == FALSE)
+	if (g_file_test (dir, G_FILE_TEST_EXISTS) &&
+	    !g_file_test (dir, G_FILE_TEST_IS_DIR))
 	{
-		if (g_file_test (dir, G_FILE_TEST_EXISTS) == TRUE)
-		{
-			g_warning (_("\"%s\" exists, please move it out of the way."), dir);
-			return FALSE;
-		}
+		g_set_error (error,
+			     EPHY_FILE_HELPERS_ERROR_QUARK,
+			     0,
+			     _("The file “%s” exists. Please move it out of the way."),
+			     dir);
+		return FALSE;
+	}
 
-		if (mkdir (dir, 488) != 0)
-		{
-			g_warning (_("Failed to create directory \"%s\"."), dir);
-			return FALSE;
-		}
+	if (!g_file_test (dir, G_FILE_TEST_EXISTS) &&
+            mkdir (dir, 488) != 0)
+	{
+		g_set_error (error,
+			     EPHY_FILE_HELPERS_ERROR_QUARK,
+			     0,
+			     _("Failed to create directory “%s”."),
+			     dir);
+		return FALSE;
 	}
 
 	return TRUE;
@@ -433,8 +488,7 @@ failed:
 void
 ephy_file_delete_on_exit (const char *path)
 {
-	del_on_exit = g_list_prepend (del_on_exit,
-				      g_strdup (path));
+	/* does nothing now */
 }
 
 static void
@@ -955,8 +1009,6 @@ ephy_file_launch_handler (const char *mime_type,
 	return ret;
 }
 
-#define DELAY_MAX_TICKS	64
-
 struct _EphyFileMonitor
 {
 	GnomeVFSMonitorHandle *handle;
@@ -967,6 +1019,7 @@ struct _EphyFileMonitor
 	guint delay;
 	guint timeout_id;
 	guint ticks;
+	GnomeVFSMonitorEventType type;
 };
 
 static gboolean
@@ -991,7 +1044,7 @@ ephy_file_monitor_timeout_cb (EphyFileMonitor *monitor)
 
 	monitor->timeout_id = 0;
 
-	monitor->callback (monitor, monitor->uri, monitor->user_data);
+	monitor->callback (monitor, monitor->uri, monitor->type, monitor->user_data);
 
 	/* don't run again */
 	return FALSE;
@@ -1009,12 +1062,16 @@ ephy_file_monitor_cb (GnomeVFSMonitorHandle *handle,
 	switch (event_type)
 	{
 		case GNOME_VFS_MONITOR_EVENT_CHANGED:
+			monitor->ticks = INITIAL_TICKS;
+			/* fall-through */
 		case GNOME_VFS_MONITOR_EVENT_CREATED:
 			/* We make a lot of assumptions here, but basically we know
 			 * that we just have to reload, by construction.
 			 * Delay the reload a little bit so we don't endlessly
 			 * reload while a file is written.
 			 */
+			monitor->type = event_type;
+
 			if (monitor->ticks == 0)
 			{
 				monitor->ticks = 1;
@@ -1037,6 +1094,15 @@ ephy_file_monitor_cb (GnomeVFSMonitorHandle *handle,
 			break;
 
 		case GNOME_VFS_MONITOR_EVENT_DELETED:
+			if (monitor->timeout_id != 0)
+			{
+				g_source_remove (monitor->timeout_id);
+				monitor->timeout_id = 0;
+			}
+			monitor->ticks = 0;
+
+			monitor->callback (monitor, monitor->uri, event_type, monitor->user_data);
+			break;
 		case GNOME_VFS_MONITOR_EVENT_STARTEXECUTING:
 		case GNOME_VFS_MONITOR_EVENT_STOPEXECUTING:
 		case GNOME_VFS_MONITOR_EVENT_METADATA_CHANGED:
@@ -1102,3 +1168,9 @@ ephy_file_monitor_cancel (EphyFileMonitor *monitor)
 	g_free (monitor->uri);
 	g_free (monitor);
 }
+
+void
+ephy_file_delete_directory (const char *path)
+{
+	/* FIXME not implemented yet */
+}
