@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-bookmarks-import.c,v 1.44 2005/10/18 20:53:55 chpe Exp $
+ *  $Id: ephy-bookmarks-import.c,v 1.45 2005/11/10 21:32:36 crispin Exp $
  */
 
 #include "config.h"
@@ -665,7 +665,7 @@ ephy_bookmarks_import_mozilla (EphyBookmarks *bookmarks,
 {
 	FILE *bf;  /* bookmark file */
 	GString *name, *url;
-	char *parsedname, *topic;
+	char *parsedname;
 	GList *folders = NULL;
 
 	if (eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_BOOKMARK_EDITING)) return FALSE;
