@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-main.c,v 1.54 2006/01/30 21:58:56 chpe Exp $
+ *  $Id: ephy-main.c,v 1.62 2006/02/13 23:01:21 chpe Exp $
  */
 
 #include "config.h"
@@ -32,6 +32,7 @@
 #include "ephy-activation.h"
 #include "ephy-session.h"
 #include "ephy-shell.h"
+#include "ephy-prefs.h"
 #include "ephy-debug.h"
 
 #include <libxml/xmlversion.h>
@@ -44,7 +45,6 @@
 #include <gtk/gtkmessagedialog.h>
 
 #include <libgnome/gnome-program.h>
-#include <libgnomeui/gnome-client.h>
 #include <libgnomeui/gnome-ui-init.h>
 
 #include <libgnomevfs/gnome-vfs-init.h>
@@ -60,13 +60,17 @@ static GQuark startup_error_quark = 0;
 static gboolean open_in_new_tab = FALSE;
 static gboolean open_in_new_window = FALSE;
 static gboolean open_as_bookmarks_editor = FALSE;
-static gboolean private_instance = FALSE;
 //static gboolean reload_plugins = FALSE;
 
 static char *session_filename = NULL;
 static char *bookmark_url = NULL;
 static char *bookmarks_file = NULL;
-static char **remaining_arguments = NULL;
+static char **arguments = NULL;
+
+/* Only set from options in debug builds */
+static gboolean private_instance = FALSE;
+static gboolean keep_temp_directory = FALSE;
+static char *profile_directory = NULL;
 
 static const GOptionEntry option_entries[] =
 {
@@ -84,7 +88,9 @@ static const GOptionEntry option_entries[] =
 	  N_("Add a bookmark"), N_("URL") },
 	{ "private-instance", 0, 0, G_OPTION_ARG_NONE, &private_instance,
 	  N_("Start a private instance"), NULL },
-	{ G_OPTION_REMAINING, '\0', 0, G_OPTION_ARG_STRING_ARRAY, &remaining_arguments,
+	{ "profile", 0, 0, G_OPTION_ARG_STRING, &profile_directory,
+	  N_("Profile directory to use in the private instance"), N_("DIR") },
+	{ G_OPTION_REMAINING, '\0', 0, G_OPTION_ARG_FILENAME_ARRAY, &arguments,
 	  "", "" },
 	{ NULL }
 };
@@ -113,8 +119,8 @@ static const GOptionEntry libgnome_option_entries[] =
 #ifdef GNOME_ENABLE_DEBUG
 static GOptionEntry debug_option_entries[] =
 {
-	{ "private-instance", 0, 0, G_OPTION_ARG_NONE, &private_instance,
-	  N_("Start a private instance"), NULL },
+	{ "keep-tempdir", 0, 0, G_OPTION_ARG_NONE, &keep_temp_directory,
+	  "Don't delete the temporary directory on exit", NULL },
 	{ NULL }
 };
 #endif /* GNOME_ENABLE_DEBUG */
@@ -235,117 +241,6 @@ shell_weak_notify (gpointer data,
 	}
 }
 
-static void
-dbus_g_proxy_finalized_cb (EphyShell *shell,
-			   GObject *zombie)
-{
-	LOG ("dbus_g_proxy_finalized_cb");
-
-	g_object_unref (shell);
-}
-
-/* Gnome session client */
-
-static gboolean
-save_yourself_cb (GnomeClient *client,
-		  gint phase,
-		  GnomeSaveStyle save_style,
-		  gboolean shutdown,
-		  GnomeInteractStyle interact_style,
-		  gboolean fast,
-		  gpointer user_data)
-{
-	EphyShell *shell;
-	EphySession *session;
-	char *argv[] = { NULL, "--load-session", NULL };
-	char *discard_argv[] = { "rm", "-f", NULL };
-	char *tmp, *save_to;
-
-	LOG ("save_yourself_cb");
-
-	/* FIXME FIXME */
-	if (!ephy_shell_get_default ()) return FALSE;
-
-	tmp = g_build_filename (ephy_dot_dir (),
-				"session_gnome-XXXXXX",
-				NULL);
-	save_to = ephy_file_tmp_filename (tmp, "xml");
-	g_free (tmp);
-
-	shell = ephy_shell_get_default ();
-	g_assert (shell != NULL);
-
-	session = EPHY_SESSION (ephy_shell_get_session (shell));
-	g_assert (session != NULL);
-
-	argv[0] = g_get_prgname ();
-	argv[2] = save_to;
-	gnome_client_set_restart_command
-		(client, 3, argv);
-
-	discard_argv[2] = save_to;
-	gnome_client_set_discard_command (client, 3,
-					  discard_argv);
-
-	ephy_session_save (session, save_to);
-
-	g_free (save_to);
-
-	return TRUE;
-}
-
-static void
-die_cb (GnomeClient* client,
-	gpointer user_data)
-	
-{
-	EphyShell *shell;
-	EphySession *session;
-
-	LOG ("die_cb");
-
-	/* FIXME FIXME */
-	if (!ephy_shell_get_default ()) return;
-
-	shell = ephy_shell_get_default ();
-	g_assert (shell != NULL);
-
-	session = EPHY_SESSION (ephy_shell_get_session (shell));
-	g_assert (session != NULL);
-
-	ephy_session_close (session);
-}
-
-static void
-gnome_session_init (void)
-{
-	GnomeClient *client;
-
-	client = gnome_master_client ();
-
-	g_signal_connect (client, "save_yourself",
-			  G_CALLBACK (save_yourself_cb), NULL);
-	g_signal_connect (client, "die",
-			  G_CALLBACK (die_cb), NULL);
-}
-
-#if 0
-static char *
-path_from_command_line_arg (const char *arg)
-{
-	char path[PATH_MAX];
-
-	if (realpath (arg, path) != NULL)
-	{
-		return g_strdup (path);
-	}
-	else
-	{
-		return g_strdup (arg);
-	}
-}
-#endif
-
 static void
 unref_proxy_reply_cb (DBusGProxy *proxy,
 		      GError *error,
@@ -354,12 +249,12 @@ unref_proxy_reply_cb (DBusGProxy *proxy,
 	if (error != NULL)
 	{
 		g_warning ("An error occured while calling remote method: %s", error->message);
-		g_error_free (error);
+		g_error_free (error);// FIXME???
 	}
 
 	g_object_unref (proxy);
 
-	if (!_ephy_dbus_is_name_owner () && gtk_main_level ())
+	if (gtk_main_level ())
 	{
 		gtk_main_quit ();
 	}
@@ -370,49 +265,42 @@ open_urls (DBusGProxy *proxy,
 	   guint32 user_time,
 	   GError **error)
 {
-	const char *options = "new-window";
-	int i;
+	static const char *empty_arguments[] = { "", NULL };
+	GString *options;
+	char **uris;
+
+	options = g_string_sized_new (64);
 
 	if (open_in_new_window)
 	{
-		options = "new-window";
+		g_string_append (options, "new-window,");
 	}
-	else if (open_in_new_tab)
+	if (open_in_new_tab)
 	{
-		options = "new-tab";
+		g_string_append (options, "new-tab,");
 	}
 
-	if (remaining_arguments == NULL)
+	if (arguments == NULL)
 	{
-		/* Homepage or resume */
-		org_gnome_Epiphany_load_url_async
-			(proxy, "", options, user_time,
-			 unref_proxy_reply_cb, NULL /* FIXME! */);
+		uris = (char **) empty_arguments;
 	}
 	else
 	{
-		for (i = 0; remaining_arguments[i] != NULL; ++i)
-		{
-			char *path;
-
-			path = remaining_arguments[i];
-			/* path = path_from_command_line_arg (args[i]); */
-
-			g_object_ref (proxy);
-
-			org_gnome_Epiphany_load_url_async
-				(proxy, path, options, user_time,
-				 unref_proxy_reply_cb, NULL /* FIXME */);
-
-			/* g_free (path); */
-		}
-
-		g_strfreev (remaining_arguments);
-		remaining_arguments = NULL;
-
-		ephy_object_idle_unref (proxy);
+		uris = (char **) arguments;
 	}
 
+	org_gnome_Epiphany_load_ur_ilist_async
+		(proxy, (const char **) uris, options->str, user_time,
+		 unref_proxy_reply_cb, NULL);
+	
+	if (arguments != NULL)
+	{
+		g_strfreev (arguments);
+		arguments = NULL;
+	}
+
+	g_string_free (options, TRUE);
+
 	return TRUE;
 }
 
@@ -437,22 +325,13 @@ call_dbus_proxy (DBusGProxy *proxy,
 		org_gnome_Epiphany_load_session_async
 			(proxy, session_filename, user_time,
 			 unref_proxy_reply_cb, shell);
+
+		g_free (session_filename);
+		session_filename = NULL;
 	}
 	else
 	{
-		/* no need to open the homepage if autoresume returns TRUE;
-		 * we already opened session windows */
-		if (!_ephy_dbus_is_name_owner () ||
-		    (ephy_session_autoresume
-			(EPHY_SESSION (ephy_shell_get_session (shell)),
-			 user_time) == FALSE))
-		{
-			retval = open_urls (proxy, user_time, error);
-		}
-		else
-		{
-			ephy_object_idle_unref (proxy);
-		}
+		retval = open_urls (proxy, user_time, error);
 	}
 
 	/* FIXME why? */
@@ -461,6 +340,70 @@ call_dbus_proxy (DBusGProxy *proxy,
 	return retval;
 }
 
+static void
+queue_commands (guint32 user_time)
+{
+	EphyShell *shell;
+	EphySession *session;
+
+	shell = ephy_shell_get_default ();
+	g_assert (shell != NULL);
+
+	session = EPHY_SESSION (ephy_shell_get_session (shell));
+	g_assert (session != NULL);
+
+	/* We only get here when starting a new instance, so we 
+	 * first need to autoresume!
+	 */
+	ephy_session_queue_command (session,
+				    EPHY_SESSION_CMD_RESUME_SESSION,
+				    NULL, NULL, user_time, TRUE);
+
+	if (open_as_bookmarks_editor)
+	{
+		ephy_session_queue_command (session,
+					    EPHY_SESSION_CMD_OPEN_BOOKMARKS_EDITOR,
+					    NULL, NULL, user_time, FALSE);
+	}
+	else if (session_filename != NULL)
+	{
+		ephy_session_queue_command (session,
+					    EPHY_SESSION_CMD_LOAD_SESSION,
+					    session_filename, NULL,
+					    user_time, FALSE);
+
+		g_free (session_filename);
+		session_filename = NULL;
+	}
+	/* Don't queue any window openings if no extra arguments given,
+	 * since session autoresume will open one for us.
+	 */
+	else if (arguments != NULL)
+	{
+		GString *options;
+
+		options = g_string_sized_new (64);
+
+		if (open_in_new_window)
+		{
+			g_string_append (options, "new-window,");
+		}
+		if (open_in_new_tab)
+		{
+			g_string_append (options, "new-tab,");
+		}
+
+		ephy_session_queue_command (session,
+					    EPHY_SESSION_CMD_OPEN_URIS,
+					    options->str,
+					    arguments,
+					    user_time, FALSE);
+
+		g_strfreev (arguments);
+		arguments = NULL;
+	}
+}
+
 static void
 show_error_message (GError **error)
 {
@@ -554,6 +497,8 @@ main (int argc,
 
 	if (!g_option_context_parse (option_context, &argc, &argv, &error))
 	{
+		g_option_context_free (option_context);
+
 		g_print ("%s\n", error->message);
 		g_error_free (error);
 		exit (1);
@@ -597,6 +542,75 @@ main (int argc,
 
 #endif /* GNOME_PARAM_GOPTION_CONTEXT */
 
+	/* Some argument sanity checks*/
+	if (arguments != NULL && (session_filename != NULL || open_as_bookmarks_editor))
+	{
+		g_print ("Cannot use --bookmarks-editor or --load-session with URL arguments\n");
+		exit (1);
+	}
+
+	if (profile_directory != NULL && private_instance == FALSE)
+	{
+		g_print ("--profile can only be used in combination with --private-instance\n");
+		exit (1);
+	}
+
+	if (arguments != NULL &&
+	    eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_ARBITRARY_URL))
+	{
+		g_print ("URL loading is locked down\n");
+		exit (1);
+	}
+
+	/* Make URIs from arguments, to support filename args */
+	if (arguments != NULL)
+	{
+		guint i;
+
+		for (i = 0; arguments[i] != NULL; ++i)
+		{
+			char *uri, *path = NULL;
+
+			path = realpath (arguments[i], NULL);
+			if (path != NULL)
+			{
+				uri = g_locale_to_utf8 (path, -1, 
+							NULL, NULL, &error);
+				free (path);
+			}
+			else
+			{
+				uri = g_locale_to_utf8 (arguments[i], -1, 
+							NULL, NULL, &error);
+			}
+
+			if (uri != NULL)
+			{
+				g_free (arguments[i]);
+
+				/* If it's a file, use gnome_vfs_make_uri_from_shell_arg,
+				 * so we get the right escaping.
+				 */
+				if (path != NULL)
+				{
+					arguments[i] = gnome_vfs_make_uri_from_shell_arg (uri);
+					g_free (uri);
+				}
+				else
+				{
+					arguments[i] = uri;
+				}
+			}
+			else
+			{
+				g_print ("Could not convert '%s' to UTF-8: %s!\n",
+					 arguments[i], error->message);
+				g_error_free (error);
+				exit (1);
+			}
+		}
+	}
+
 	/* Get a timestamp manually if need be */
 	if (user_time == 0)
 	{
@@ -611,7 +625,7 @@ main (int argc,
 
 	startup_error_quark = g_quark_from_static_string ("epiphany-startup-error");
 
-	if (!_ephy_dbus_startup (&error))
+	if (!_ephy_dbus_startup (!private_instance, &error))
 	{
 		_ephy_dbus_release ();
 
@@ -620,26 +634,27 @@ main (int argc,
 		exit (1);
 	}
 
-	/* Create DBUS proxy */
-	proxy = ephy_dbus_get_proxy (ephy_dbus_get_default (), EPHY_DBUS_SESSION);
-	if (proxy == NULL)
-	{
-		error = g_error_new (STARTUP_ERROR_QUARK,
-				     0,
-				     "Unable to get DBus proxy; aborting activation."); /* FIXME i18n */
-
-		_ephy_dbus_release ();
-
-		show_error_message (&error);
-
-		exit (1);
-	}
-
 	/* If we're remoting, no need to start up any further services,
 	 * just forward the call.
 	 */
-	if (!_ephy_dbus_is_name_owner ())
+	if (!private_instance &&
+	    !_ephy_dbus_is_name_owner ())
 	{
+		/* Create DBUS proxy */
+		proxy = ephy_dbus_get_proxy (ephy_dbus_get_default (), EPHY_DBUS_SESSION);
+		if (proxy == NULL)
+		{
+			error = g_error_new (STARTUP_ERROR_QUARK,
+					     0,
+					     "Unable to get DBus proxy; aborting activation."); /* FIXME i18n */
+
+			_ephy_dbus_release ();
+
+			show_error_message (&error);
+
+			exit (1);
+		}
+
 		if (!call_dbus_proxy (proxy, user_time, &error))
 		{
 			_ephy_dbus_release ();
@@ -660,7 +675,10 @@ main (int argc,
 
 	/* We're not remoting; start our services */
 
-	if (!ephy_file_helpers_init (FALSE, &error))
+	if (!ephy_file_helpers_init (profile_directory,
+				     private_instance,
+				     keep_temp_directory,
+				     &error))
 	{
 		_ephy_dbus_release ();
 
@@ -669,9 +687,6 @@ main (int argc,
 		exit (1);
 	}
 
-	/* init the session manager up here so we can quit while the resume dialogue is shown */
-	gnome_session_init ();
-
 	eel_gconf_monitor_add ("/apps/epiphany/general");
 	gnome_vfs_init ();
 	ephy_stock_icons_init ();
@@ -691,23 +706,8 @@ main (int argc,
 
 	/* Now create the shell */
 	_ephy_shell_create_instance ();
-	g_object_weak_ref (G_OBJECT (proxy),
-			   (GWeakNotify) dbus_g_proxy_finalized_cb,
-			   g_object_ref (ephy_shell_get_default ()));
 
-	if (!call_dbus_proxy (proxy, user_time, &error))
-	{
-		g_object_unref (ephy_shell_get_default ());
-		g_object_unref (proxy);
-		g_assert (ephy_shell_get_default () == NULL);
-
-		ephy_file_helpers_shutdown ();
-		_ephy_dbus_release ();
-
-		show_error_message (&error);
-
-		exit (1);
-	}
+	queue_commands (user_time);
 
 	/* We'll release the initial reference on idle */
 	g_object_weak_ref (G_OBJECT (ephy_shell), shell_weak_notify, NULL);
