@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-shell.c,v 1.139 2006/01/25 23:34:37 chpe Exp $
+ *  $Id: ephy-shell.c,v 1.141 2006/02/21 13:44:16 paharvey Exp $
  */
 
 #include "config.h"
@@ -668,6 +668,7 @@ ephy_shell_get_toolbars_model (EphyShell *shell, gboolean fullscreen)
 	{
 		if (shell->priv->fs_toolbars_model == NULL)
 		{
+			EggTbModelFlags flags;
 			gboolean success;
 			const char *xml;
 
@@ -675,9 +676,15 @@ ephy_shell_get_toolbars_model (EphyShell *shell, gboolean fullscreen)
 			xml = ephy_file ("epiphany-fs-toolbar.xml");
 			g_return_val_if_fail (xml != NULL, NULL);
 
-			success = egg_toolbars_model_load
+			success = egg_toolbars_model_load_toolbars
 				(shell->priv->fs_toolbars_model, xml);
 			g_return_val_if_fail (success, NULL);
+
+			flags = egg_toolbars_model_get_flags 
+			  (shell->priv->fs_toolbars_model, 0);
+			egg_toolbars_model_set_flags
+			  (shell->priv->fs_toolbars_model, 0, 
+			   flags | EGG_TB_MODEL_NOT_REMOVABLE);
 		}
 
 		return G_OBJECT (shell->priv->fs_toolbars_model);
