@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: popup-commands.c,v 1.59 2006/01/27 22:14:44 paharvey Exp $
+ *  $Id: popup-commands.c,v 1.60 2006/02/04 09:26:09 paharvey Exp $
  */
 
 #include "config.h"
@@ -127,7 +127,7 @@ popup_cmd_bookmark_link (GtkAction *action,
 		location = rel;
 	}
 
-	ephy_bookmarks_ui_add_bookmark (location, title);
+	ephy_bookmarks_ui_add_bookmark (GTK_WINDOW (window), location, title);
 }
 
 static void
